import sys
import paramiko
import json
import configparser
import os
import subprocess
import time
sys.path.append('../common')
from runlogger import *

logDir="/store/var/log"
def runTest(test_id,log_dir,dir):
    esx_host = getenvdata(dir,'esx','hostname')[0]
    esx_user= getenvdata(dir,'esx','user')[0]
    esx_password = getenvdata(dir,'esx','password')[0]
    resultfile = open(log_dir+'/.result', 'w')
    testresult = False
    ssh = sshConnection(esx_host,esx_user,esx_password,resultfile)
    #Evaluating the ESXi host Version to install the right vib if required
    command = "vmware -v"
    stdin, stdout, stderr = ssh.exec_command(command)
    while True:
        nextline = stdout.readline()
        if nextline == '':
            break
        logging.info(nextline)
        st=nextline.split(' ')
    logging.info("ESXi Version being Certified is "+st[2])
    # Is memcert.vib installed on the host?
    command = "esxcli software vib list | grep memcert"
    stdin, stdout, stderr = ssh.exec_command(command)
    lst = []
    while True:
        nextline = stdout.readline()
        if nextline == '':
            break
        logging.info(nextline)
        lst.append(nextline)
    str1=''.join(lst)
    if 'memcert' in str1:
        logging.info("The memcert vib is already installed on the host "+esx_host)
    else:
        # Upload and install memcert.vib
        logging.info("Start Copying the memcert vib to "+esx_host+" \n")
        sftp = ssh.open_sftp()
        sftp.put("/certs/flash-cert/FlashMemory/"+st[2]+"/memcert.vib", "/memcert.vib")
        sftp.close()
        logging.info("Copying of memcert vib done..\n")
        command = "esxcli software vib install -v /memcert.vib --no-sig-check"
        stdin, stdout, stderr = ssh.exec_command(command)
        list2 = []
        while True:
            nextline = stdout.readline()
            if nextline == '':
                break
            logging.info(nextline)
            list2.append(nextline)
        str2=''.join(list2)
        if 'Message: Operation finished successfully' in str2:
            logging.info("memcert.vib installed successfully")
        else:
            logging.info("Failed to install memcert.vib")
    executeTest(test_id,esx_host,esx_user,esx_password,resultfile)
    monitorTest(esx_host,esx_user,esx_password,resultfile)
    testresult =  analyzeTest(esx_host,esx_user,esx_password,resultfile)
    cleanup(esx_host,esx_user,esx_password,resultfile,log_dir)
    if testresult == True:
        resultfile.write('Passed')
        logging.info('Passed')
    else:
        resultfile.write('Failed')
        logging.info('Failed')
    resultfile.close
    ssh.close()

def analyzeTest(esx_host,esx_user,esx_password,resultfile):
    ssh = sshConnection(esx_host,esx_user,esx_password,resultfile)
    command = "cat "+logDir+"/memcert-state"
    stdin, stdout, stderr = ssh.exec_command(command)
    while True:
        nextline = stdout.readline()
        if nextline == '':
            break
        logging.info(nextline)
        if '110' in nextline:
            testresult = True
            logging.info("Test Passed")
            return testresult
        elif '120' in nextline:
            testresult = False
            logging.info("Test Failed")
            return testresult
        elif '130' in nextline:
            testresult = False
            logging.info("Test Failed")
            return testresult
        else:
            testresult = False
            logging.info("Result unknown")
            return testresult
    ssh.close()

def cleanup(esx_host,esx_user,esx_password,resultfile,log_dir):
    ssh = sshConnection(esx_host,esx_user,esx_password,resultfile)
    status = checkStatus(esx_host,esx_user,esx_password,resultfile)
    if 'is running' in status:
        command = "/etc/init.d/memcert stop"
        stdin, stdout, stderr = ssh.exec_command(command)
    logging.info("Start Copying the logs from "+esx_host+" to local\n")
    sftp = ssh.open_sftp()
    sftp.get(logDir+"/esxitest.log", log_dir+"/esxitest.log")
    sftp.close()
    ssh.close()
    logging.info("Copying of logs from "+esx_host+" to local is successful \n")

def monitorTest(esx_host,esx_user,esx_password,resultfile):
    #ssh = sshConnection(esx_host,esx_user,esx_password,resultfile)
    while True:
        time.sleep(50)
        response = subprocess.call("ping -c 1 "+esx_host, shell=True)
        if(response != 0):
            logging.info("Sleep for 500 seconds....")
            time.sleep(500)
            ssh = sshConnection(esx_host,esx_user,esx_password,resultfile)
            command = "/etc/init.d/memcert status"
            stdin, stdout, stderr = ssh.exec_command(command)
            while True:
                nextline1 = stdout.readline()
                if nextline1 == '':
                    break
                logging.info(nextline1)
                if 'not running' in nextline1:
                    command = "/etc/init.d/memcert start"
                    stdin, stdout, stderr = ssh.exec_command(command)
            time.sleep(15)
            ssh.close()
        status = checkStatus(esx_host,esx_user,esx_password,resultfile)
        if 'not running' in status:
            logging.info("Status says not running.............")
            break

def checkStatus(esx_host,esx_user,esx_password,resultfile):
    ssh = sshConnection(esx_host,esx_user,esx_password,resultfile)
    command = "/etc/init.d/memcert status"
    stdin, stdout, stderr = ssh.exec_command(command)
    list2 = []
    while True:
        nextline1 = stdout.readline()
        if nextline1 == '':
            break
        logging.info(nextline1)
        list2.append(nextline1)
    str3=''.join(list2)
    ssh.close()
    return str3


def executeTest(test_id,esx_host,esx_user,esx_password,resultfile):
    ssh = sshConnection(esx_host,esx_user,esx_password,resultfile)
    command = "/etc/init.d/memcert status"
    stdin, stdout, stderr = ssh.exec_command(command)
    while True:
        nextline1 = stdout.readline()
        if nextline1 == '':
            break
        logging.info(nextline1)
        if 'not running' in nextline1:
            command = "rm -f "+logDir+"/memcert-state"
            stdin, stdout, stderr = ssh.exec_command(command)
            while True:
                nextline = stdout.readline()
                if nextline == '':
                    break
                logging.info(nextline)
            command = "/etc/init.d/memcert start"
            stdin, stdout, stderr = ssh.exec_command(command)
            while True:
                nextline = stdout.readline()
                if nextline == '':
                    break
                logging.info(nextline)
        else:
            logging.info("memcert is running")
    logging.info("test id is "+test_id)
    if (test_id== "flashDurability"):
        command = "echo 10 > "+logDir+"/memcert-state"
    else:
        command = "echo 2 > "+logDir+"/memcert-state"
    stdin, stdout, stderr = ssh.exec_command(command)
    sys.stdout.write(stdout.readline()),
    sys.stdout.flush()
    ssh.close()
    logging.info("ExecuteTest: Started the test on "+esx_host)

def sshConnection(esx_host,esx_user,esx_password,resultfile):
    i = 1
    while True:
        try:
            logging.info("sshConnection..")
            ssh = paramiko.SSHClient()
            ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            ssh.connect(hostname=esx_host, username=esx_user, password=esx_password)
            return ssh
        except Exception as e:
            logging.info("Error: " +str(e))
            i += 1
            time.sleep(10)
            response = subprocess.call("ping -c 1 "+esx_host, shell=True)
            if(response != 0):
                logging.info("Sleep for 500 seconds....")
                time.sleep(500)
            
            if i == 30:
                ssh.close()
                sys.exit(1)    

def getenvdata(dir, hosttype,key):
      with open(dir+'runlist.json') as json_data:
          data = json.load(json_data)
          return  [ li[key] for li in data['environment']['hosts'] if li['hosttype'] == hosttype]

def main():
    runlogger(sys.argv[2])
    runTest(sys.argv[1],sys.argv[2],'/')

if __name__== '__main__':
    main()

