import sys
import paramiko
import json
import configparser
import os
import subprocess
import time
import re
sys.path.append('../common')
from runlogger import *

#The method checks if the USB key and the controller attached in the server supports 3.0
def runTest(test_name,log_dir,dir):
    try:
        esx_host = getenvdata(dir,'esx','hostname')[0]
        esx_user = getenvdata(dir,'esx','user')[0]
        esx_password = getenvdata(dir,'esx','password')[0]
        usbVendorName = getparams(dir,test_name,'VENDORDEVNAME')
        vendorCtrlName = getparams(dir,test_name,'VENDORCTRLNAME')
        resultfile = open(log_dir+'/.result', 'w')
        testresult = False
        #Checks if the device supports USB 3.0 or not
        logging.info("ExecuteTest (First Part): Started the Device(USB Key) Test on "+esx_host+"\n")
        #sys.stdout.write("ExecuteTest (First Part): Started the Device(USB Key) Test on "+esx_host+"\n")
        #runlogfile.write("ExecuteTest (First Part): Started the Device(USB Key) Test on "+esx_host+"\n")
        ssh = sshConnection(esx_host,esx_user,esx_password,resultfile)
        command = "lsusb -v | grep -A 4 -i "+usbVendorName
        stdin, stdout, stderr = ssh.exec_command(command)
        lst = []
        while True:
            nextline = stdout.readline()
            if nextline == '':
                break
            logging.info(nextline)
            lst.append(nextline)
        str1 = ''.join(lst)
        m = re.search('.*\s+bcdUSB\s+3(.*)' , str1)
        logging.info("m value ")
        logging.info(m)
        if m:
            logging.info("The USB Key Belonging to vendor : "+usbVendorName+" supports USB 3"+m.group(1)+" \n")
            testresult = True
            usbControllerTest(vendorCtrlName,esx_host,esx_user,esx_password,resultfile)
        else:
            logging.info("The USB Key Belonging to vendor : "+usbVendorName+" does not supports USB 3.00 \n")
            testresult = False
        if testresult == True:
            logging.info('Passed')
            resultfile.write('Passed')
        else:
            logging.info('Failed')
            resultfile.write('Failed')
        resultfile.close
        ssh.close()
    except Exception as e:
        logging.info("Error: " +str(e))
        resultfile.write("Failed")
        resultfile.close
        ssh.close()
        sys.exit(1)


def usbControllerTest(vendorCtrlName,esx_host,esx_user,esx_password,resultfile):
    ssh = sshConnection(esx_host,esx_user,esx_password,resultfile)
    logging.info("ExecuteTest (Second Part): Started the USB Controller Test on "+esx_host+"\n")
    command = "lspci -d | grep '30[ ]03[ ]0c'"
    stdin, stdout, stderr = ssh.exec_command(command)
    lists = []
    while True:
        nextline = stdout.readline()
        if nextline == '':
            break
        logging.info(nextline)
        lists.append(nextline)
    str2 = ''.join(lists)
    if re.search('(.*30 03 0c.*)' , str2):
        logging.info("The controller "+vendorCtrlName+" supports USB 3.0 \n")
        testresult = True
    else:
        logging.info("The controller "+vendorCtrlName+" does not support USB 3.0 or not found in the attached devices list")
        testresult = False

def sshConnection(esx_host,esx_user,esx_password,resultfile):
    try:
        logging.info("sshConnection..")
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=esx_host, username=esx_user, password=esx_password)
        return ssh
    except Exception as e:
        logging.info("Error: " +str(e))
        resultfile.write("Failed")
        resultfile.close
        ssh.close()
        sys.exit(1)

def getenvdata(dir, hosttype,key):
      with open(dir+'runlist.json') as json_data:
          data = json.load(json_data)
          return  [ li[key] for li in data['environment']['hosts'] if li['hosttype'] == hosttype]

def getparams(dir,test_name,key):
    with open('/runlist.json') as json_data:
        data = json.load(json_data)
        #return  data['params'][key]
        for item in [li['params'] for li in data['params']['testParams'] if li['name'] == test_name]:
            if key in item:
               return item[key]
            else:
               item[key]="NA"
               return item[key]
def main():
    runlogger(sys.argv[2])
    runTest(sys.argv[1],sys.argv[2],'/')

if __name__== '__main__':
    main()
